import logging
import sys
from logging import getLogger

from myver.cli import cli_entry
from myver.error import MyverError


def main():
    logging.basicConfig(
        format='[%(levelname)s] %(message)s',
        level=logging.WARNING
    )
    log = getLogger(__name__)
    try:
        cli_entry()
    except KeyboardInterrupt:
        sys.exit(1)
    except MyverError as e:
        log.error(e.message)
        sys.exit(1)
    except Exception as e:
        log.error('MyVer failed', exc_info=e)
        sys.exit(1)


if __name__ == '__main__':
    main()



import argparse
import logging
import textwrap

from myver.config import Config
from myver.error import MyverError


def cli_entry(input_args=None):
    """Entry point for the command line utility."""
    args = _parse_args(input_args)

    if args.help:
        print(textwrap.dedent('''\
        Usage: myver [OPTIONS]
        
        Options:
          -h, --help               Show this help message and exit
          -b, --bump strings       Bump version parts
              --config string      Config file path
          -c, --current [strings]  Get the current version or version parts
          -r, --reset strings      Reset version parts
          -v, --verbose            Log more details
        ''').rstrip())
        return

    # Running these before setting up config so that all logging can
    # be captured effectively.
    _handle_verbose(args)
    _handle_debug(args)

    config = Config(args.config)

    # Most things after here will need the config.
    _handle_current(args, config)
    _handle_bump(args, config)
    _handle_reset(args, config)


def _handle_verbose(args):
    if args.verbose:
        logging.root.setLevel(logging.INFO)


def _handle_debug(args):
    if args.debug:
        logging.root.handlers[0].setFormatter(
            logging.Formatter('[%(levelname)s] [%(module)s] %(message)s')
        )
        logging.root.setLevel(logging.DEBUG)


def _handle_current(args, config: Config):
    if args.current is None:
        return

    if len(args.current) == 0:
        print(config.version)
    else:
        try:
            print(config.version.parse(args.current))
        except KeyError as key_error:
            raise MyverError(f'Failed to parse --current option, contains '
                             f'invalid part key `{key_error.args[0]}`')


def _handle_bump(args, config: Config):
    _do_update(config.version.bump, args.bump, config)


def _handle_reset(args, config: Config):
    _do_update(config.version.reset, args.reset, config)


def _do_update(func, arg, config: Config):
    if arg:
        old_version_str = str(config.version)
        func(arg)
        new_version_str = str(config.version)
        config.save()
        config.update_files(old_version_str, new_version_str)
        print(f'{old_version_str}  >>  {new_version_str}')


def _parse_args(args):
    parser = argparse.ArgumentParser(
        prog='myver',
        add_help=False,
    )
    parser.register('action', 'extend', ExtendAction)
    parser.add_argument(
        '-h', '--help',
        action='store_true',
    )
    parser.add_argument(
        '-b', '--bump',
        action='extend',
        nargs='+',
        type=str,
    )
    parser.add_argument(
        '--config',
        default='myver.yml',
        type=str,
    )
    parser.add_argument(
        '-c', '--current',
        action='extend',
        nargs='*',
        type=str,
    )
    parser.add_argument(
        '-r', '--reset',
        action='extend',
        nargs='+',
        type=str,
    )

    # Extra logging
    parser.add_argument(
        '-v', '--verbose',
        action='store_true',
    )
    parser.add_argument(
        '--debug',
        action='store_true',
    )
    return parser.parse_args(args)


class ExtendAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        items = getattr(namespace, self.dest) or []
        items.extend(values)
        setattr(namespace, self.dest, items)



from __future__ import annotations

import abc
from logging import getLogger
from typing import Optional, Union, List

from myver.error import ConfigError, BumpError

log = getLogger(__name__)


class Part(abc.ABC):
    """The base class for a version part.

    :param key: The unique key of the part. This is used to set dict
        keys for collections of parts.
    :param value: The actual value of the part.
    :param requires: Another part that this part requires. This means
        that the required part will need to be set if this part is set.
    """

    def __init__(self,
                 key: str,
                 value: Optional[Union[str, int]],
                 requires: Optional[str] = None,
                 prefix: Optional[str] = None,
                 child: Optional[Part] = None,
                 parent: Optional[Part] = None):
        self.prefix: str = prefix or ''
        self.key: str = key
        self.value: Optional[Union[str, int]] = value
        self.requires: Optional[str] = requires
        self._child: Optional[Part] = None
        self._parent: Optional[Part] = None
        self.child = child
        self.parent = parent

    @abc.abstractmethod
    def next_value(self, value_override: str = None):
        """Set current value to the next part value.

        :param value_override: Manual override for the bumped value.
        """

    @property
    @abc.abstractmethod
    def start(self) -> Union[str, int]:
        """Get the start value in a usable form (i.e. not None)"""

    @start.setter
    @abc.abstractmethod
    def start(self, new_start: Union[str, int]):
        """Set the start value"""

    @property
    def child(self) -> Optional[Part]:
        return self._child

    @child.setter
    def child(self, part: Optional[Part]):
        self._child = part
        if self._child and not self._child.parent:
            self._child.parent = self

    @property
    def parent(self) -> Optional[Part]:
        return self._parent

    @parent.setter
    def parent(self, part: Optional[Part]):
        self._parent = part
        if self._parent and not self._parent.child:
            self._parent.child = self

    def is_set(self) -> bool:
        """Checks if the part's value is not None."""
        return self.value is not None

    def bump(self, bump_args: List[str] = None, value_override: str = None):
        """Bump this part's value.

        :param bump_args: The bump arguments.
        :param value_override: Manual override for the bumped value.
        """
        log.info(f'Bumping <{self.key}>')
        bump_args = bump_args or []
        self.next_value(value_override)
        if self.child:
            self.child.reset(bump_args)

    def reset(self, bump_args: List[str] = None):
        """Reset part value to the start value.

        Resetting the part to the start value will also make a recursive
        call to its child, resetting their values too.

        :param bump_args: The keys that are being bumped.
        """
        log.info(f'Resetting <{self.key}>')
        bump_args = bump_args or []

        # If this part is required and it's in the bump keys, we want to
        # skip this step so that we do not get a double bump.
        if self.is_required() and self.key not in bump_args:
            self.value = self.start
        else:
            self.value = None

        if self.child:
            self.child.reset()

    def is_required(self) -> bool:
        """Checks if this part is required by any parents.

        If this part has no parent, then it is assumed to be required
        since it means that it is the head of the version.
        """
        if self.parent is None:
            return True
        return self._parent_requires(self.key)

    def _parent_requires(self, key: str) -> bool:
        """Check if a part is required based on its key.

        This does a recursive call up to all of the parents until it
        reaches the final parent to check if any of them require the
        part specified.

        :param key: The key of the part that you want to check.
        """
        if self.parent is not None:
            if self.parent.requires == key and self.parent.is_set():
                log.debug(f'Part <{key}> is required')
                return True
            else:
                return self.parent._parent_requires(key)

        log.debug(f'Part <{key}> is not required')
        return False

    def __str__(self):
        return f'{self.prefix}{self.value}'

    def __eq__(self, other: Part) -> bool:
        return (self.key == other.key) and (self.value == other.value)


class IdentifierPart(Part):
    """An identifier part.

    :param strings: List of valid strings that can be used as an
        identifier for this part.
    :param start: The starting value of the part. This is used when the
        part goes out of a null state, or is reset to its original
        state. If this is specified it must be a string that is in the
        `self.strings` List.
    """

    def __init__(self,
                 key: str,
                 value: Optional[str],
                 strings: List[str],
                 requires: Optional[str] = None,
                 prefix: Optional[str] = None,
                 child: Optional[Part] = None,
                 parent: Optional[Part] = None,
                 start: str = None):
        super().__init__(key, value, requires, prefix, child, parent)
        self._strings: List[str] = strings
        self._start: Optional[str] = start
        self.strings = strings
        self.start = start

    @property
    def strings(self):
        return self._strings

    @strings.setter
    def strings(self, new_strings: List[str]):
        self._validate_strings(new_strings)
        self._strings: List[str] = new_strings

    @property
    def start(self) -> str:
        return self._start or self.strings[0]

    @start.setter
    def start(self, new_start: str):
        self._validate_start(new_start)
        self._start = new_start

    def next_value(self, value_override: str = None):
        if value_override is not None:
            if value_override not in self.strings:
                raise BumpError(
                    f'Cannot set value override `{value_override}`, it must '
                    f'be in the strings List of part `{self.key}`')
            else:
                self.value = value_override
        elif self.is_set():
            current_index = self.strings.index(self.value)
            next_index = current_index + 1

            if next_index < len(self.strings):
                self.value = self.strings[next_index]
            else:
                self.value = None
        else:
            self.value = self.start

    def _validate_start(self, start: Optional[str]):
        if start is not None and start not in self.strings:
            raise ConfigError(
                f'Part `{self.key}` has an `identifier.start` value that is '
                f'not in the `identifier.strings` List')

    def _validate_strings(self, strings: List[str]):
        if not len(strings) > 0:
            raise ConfigError(
                f'Part `{self.key}` has an `identifier.strings` has an empty '
                f'List, the List must have at least one string')


class NumberPart(Part):
    """A number part.

    :param label_suffix: String to use for separating the label and the
        number.
    :param start: Starting value of the part. This is used when the part
        goes out of a null state, or is reset to its original state.
    :param show_start: If true, the start value will be shown in the
        version. If false, then the start value wont be shown although
        the next value (after a bump) will be shown.
    """

    def __init__(self,
                 key: str,
                 value: Optional[int],
                 requires: Optional[str] = None,
                 prefix: Optional[str] = None,
                 child: Optional[Part] = None,
                 parent: Optional[Part] = None,
                 label: Optional[str] = None,
                 label_suffix: Optional[str] = None,
                 start: int = None,
                 show_start: bool = None):
        super().__init__(key, value, requires, prefix, child, parent)
        self.label: Optional[str] = label or ''
        self.label_suffix: Optional[str] = label_suffix or ''
        if show_start is None:
            self.show_start: bool = True
        else:
            self.show_start: bool = show_start
        self._start: Optional[int] = start
        self.start = start

    @property
    def start(self) -> int:
        return self._start or 0

    @start.setter
    def start(self, new_start: int):
        self._validate_start(new_start)
        self._start = new_start

    def next_value(self, value_override: str = None):
        if value_override is not None:
            try:
                int(value_override)
            except ValueError:
                raise BumpError(
                    f'Value override `{value_override}` must be a number for '
                    f'part `{self.key}`')

            if int(value_override) < 0:
                raise BumpError(
                    f'Cannot set value override to a negative number '
                    f'`{value_override}` for part `{self.key}`')
            self.value = int(value_override)
        elif self.is_set():
            self.value = self.value + 1
        else:
            self.value = self.start

    def _validate_start(self, start: Optional[int]):
        if start is not None:
            try:
                int(start)
            except ValueError:
                raise ConfigError(
                    f'Part `{self.key}` has an invalid value for its '
                    f'`number.start` attribute, it must be an integer')

        if start is not None and start < 0:
            raise ConfigError(
                f'Part `{self.key}` has an negative value for its '
                f'`number.start` attribute, it must be positive')

    def __str__(self):
        if self.value == self.start and not self.show_start:
            return f'{self.prefix}{self.label}'
        return f'{self.prefix}{self.label}{self.label_suffix}{self.value}'



from __future__ import annotations

from logging import getLogger
from typing import List

from myver.error import ConfigError, BumpError
from myver.part import Part

log = getLogger(__name__)


class Version:
    """Represents the version itself.

    This is the top level class for a version. It contains the groups
    of parts and this is where the version operations are performed.

    :param parts: The List of parts in the version.
    """

    def __init__(self, parts: List[Part] = None):
        self._parts: List[Part] = parts or []
        self.parts = parts or []

    @property
    def parts(self) -> List[Part]:
        return self._parts

    @parts.setter
    def parts(self, new_parts: List[Part]):
        """Sets the parts List.

        :param new_parts: The parts to set.
        :raise KeyConflictError: A part key appears 2 or more times in
            the List.
        """
        validate_keys(new_parts)
        validate_requires(new_parts)
        self._parts = new_parts
        set_relationships(self._parts)

    def bump(self, args: List[str]):
        """Bump the version based on bumping args.

        :param args: The List of part keys to bump. An arg may have a
            key value pair with the syntax of `<key>=<value>`.
        :raise BumpError: When the bumping fails.
        """
        log.debug('Starting version bump')
        for arg in args:
            if arg.count('=') > 1:
                raise BumpError(
                    f'The bump arg `{arg}` contains more than 1 `=` symbol, '
                    f'there can only be 0 or 1 `=` symbol in the bump arg')
            elif arg.count('=') == 1:
                key = arg.split('=')[0]
                value_override = arg.split('=')[1]
                self.part(key).bump(args, value_override)
            else:
                self.part(arg).bump(args)

    def reset(self, keys: List[str]):
        """Reset parts based on their keys.

        :param keys: The keys of the parts to reset.
        :raise KeyError: If a key in the `keys` List does not reference
            a valid part in the version.
        """
        log.debug('Starting version reset')
        for key in keys:
            self.part(key).reset()

    def part(self, key: str) -> Part:
        """Gets a part based on its key.

        :param key: The key of the part you are getting.
        :raise KeyError: If no part has the key provided.
        """
        for part in self._parts:
            if part.key == key:
                return part
        raise KeyError(key)

    def parse(self, keys: List[str]):
        """Parses specific parts in the version.

        Lets say you parse the keys `['a', 'z']`, if `z` is not a null
        value, then all parts in between `a` and `z` will also be
        parsed, such as `b`, `c`, `d` etc... If `z` is null then only
        `a` will be parsed, and not their intermediary parts.

        :param keys: The keys of the parts to parse. They must be in
            chronological order from how they are configured.
        :raise KeyError: If an invalid part key is provided.
        """
        end_key = ''
        for key in keys:
            if self.part(key).is_set():
                end_key = key

        parsed = ''
        for part in self._parts:
            if part.is_set():
                parsed += str(part)
            if part.key == end_key:
                break

        return parsed

    def __eq__(self, other: Version):
        for this_part, other_part in zip(self.parts, other.parts):
            if not this_part == other_part:
                return False
        return True

    def __str__(self):
        version_str = ''

        for part in self._parts:
            if part.is_set():
                version_str += str(part)

        return version_str


def validate_requires(parts: List[Part]):
    """Validates that parts require other valid parts.

    :raise ConfigError: If a part requires itself or a part that does
        not exist.
    """
    keys = [p.key for p in parts] or []
    for part in parts:
        if not part.requires:
            continue

        if part.requires == part.key:
            raise ConfigError(
                f'Part `{part.key}` has a `requires` key that is'
                f'referencing itself, it must reference another part')
        if part.requires not in keys:
            raise ConfigError(
                f'Part `{part.key}` has a `requires` key '
                f'"{part.requires}" that does not exist, it must be a '
                f'valid key of another part')


def validate_keys(parts: List[Part]):
    """Validates that they keys are unique.

    :raise ConfigError: If two or more parts with the same key.
    """
    keys = [p.key for p in parts] or []
    for key in keys:
        if keys.count(key) > 1:
            raise ConfigError(
                f'Key "{key}" is configured on more than one part, all '
                f'parts must have a unique key')


def set_relationships(parts: List[Part]):
    """Sets the parent-child relationships between a List of parts.

    :param parts: The parts to set the relationships for.
    """
    for i in range(len(parts)):
        if i < len(parts) - 1:
            parts[i].child = parts[i + 1]



from logging import getLogger
from typing import List, Dict

import ruamel.yaml

from myver.error import ConfigError
from myver.files import FileUpdater
from myver.part import Part, IdentifierPart, NumberPart
from myver.version import Version

log = getLogger(__name__)


class Config:
    def __init__(self,
                 path: str = None,
                 files: List[FileUpdater] = None,
                 version: Version = None):
        self.path: str = path
        self.files: List[FileUpdater] = files
        self.version: Version = version
        if path and not (files or version):
            self.load()

    def load(self):
        """Load the config from `self.path`.

        :raise ConfigError: If the configuration file is invalid.
        """
        log.info(f'Loading config file {self.path}')
        self._load_files()
        self._load_version()

    def _load_files(self):
        config_dict = dict_from_yaml(self.path)
        self.files = files_from_dict(config_dict)

    def _load_version(self):
        config_dict = dict_from_yaml(self.path)
        self.version = version_from_dict(config_dict)

    def save(self):
        """Syncs a version to a yaml file.

        This will only sync the part values to the file, no other
        configuration of the file will change. This will mean that the
        parts in the version will need to have perfect 1:1 corresponding
        keys for each part and within the yaml file and in the `version`
        param. This also means that the yaml file must have existing
        configuration details for each part in the `version` param.

        :raise FileNotFoundError: If the file does not exist.
        :raise OSError: For other errors when accessing the file.
        :raise ConfigError: When the yaml file does not have a 1:1 of
            keys for parts compared to the `version` param.
        """
        try:
            log.info(f'Saving config file {self.path}')
            self._save_version_values()
        except KeyError as key_error:
            key = key_error.args[0]
            raise ConfigError(
                f'You must have the required attribute `{key}` configured')

    def update_files(self, old_version: str, new_version: str):
        """Update any configured files with the version."""
        for file_updater in self.files:
            file_updater.update(old_version, new_version)

    def _save_version_values(self):
        """Update config file part based on `version` object."""
        log.debug(f'Getting value update map for path {self.path}')
        update_map = self._get_value_update_map()
        log.debug(f'Update map for {self.path}:')
        log.debug(f'{update_map}')

        with open(self.path, 'r') as file:
            lines = file.readlines()

        with open(self.path, 'w') as file:
            for index, value in update_map.items():
                indent_length = len(lines[index]) - len(lines[index].lstrip())
                updated = f'{" " * indent_length}value: {value}\n'
                log.debug(f'For line {index + 1}')
                log.debug(f'Old:\n{lines[index]}')
                log.debug(f'New:\n{updated}')
                lines[index] = updated

            log.debug(f'Writing changes')
            file.writelines(lines)

    def _get_value_update_map(self) -> Dict[int, str]:
        """Get an update map for version values in a config file.

        :return: A dict with each key in the dict represents the index
            in the lines list to update (based on index counter starting
            at 0). The value of each entry in the dict represents a
            part's value to be updated at the given line index.
        """
        config_dict = dict_from_yaml(self.path)
        update_map = dict()

        with open(self.path, 'r') as file:
            lines = file.readlines()

            for key in config_dict['parts'].keys():
                log.debug(f'Getting value index for part <{key}>')
                # We want to start at the part's key so that the next
                # `value` node is guaranteed to be the part's `value` node.
                from_index = config_dict['parts'][key].lc.line
                # Note, it's an index (as related to list indexes) and not
                # a line number.
                line_index = find_value_node_index(lines, from_index)
                part = self.version.part(key)

                if part.value is None:
                    update_map[line_index] = 'null'
                else:
                    update_map[line_index] = part.value

                log.debug(f'Part <{key}> value index is <{line_index}> with '
                          f'value <{update_map[line_index]}>')

        return update_map


def dict_from_yaml(path: str) -> Dict:
    """Gets the dict config from a file.

    The default file path is `myver.yml`, which is a relative path. This
    path can be overridden by using the `path` arg.

    :param path: The path to the myver config file.
    :raise FileNotFoundError: If the file does not exist.
    :raise OSError: For other errors when accessing the file.
    """
    log.debug(f'Getting dict from yaml {path}')
    with open(path, 'r') as file:
        yaml = ruamel.yaml.YAML()
        config_dict = yaml.load(file)
    return config_dict


def find_value_node_index(lines: List[str], from_index: int) -> int:
    """Find the line index for the `value` node.

    :param lines: The lines to read from in order to get the index.
    :param from_index: The index to start searching from.
    :return: The index of where the `value` node is.
    """
    for i, line in enumerate(lines[from_index:]):
        if line.lstrip().startswith('value:'):
            return i + from_index


def version_from_dict(config_dict: Dict) -> Version:
    """Construct version from a config dict.

    :param config_dict: The dict with raw version config data.
    :raise ConfigError: If the configuration dict is invalid.
    :return: The version.
    """
    try:
        parts: List[Part] = []
        for part_key, part_dict in config_dict['parts'].items():
            parts.append(part_from_dict(part_key, part_dict))
        return Version(parts)
    except KeyError as key_error:
        key = key_error.args[0]
        raise ConfigError(
            f'You must have the required attribute `{key}` configured in '
            f'`parts`')


def part_from_dict(key: str, config_dict: Dict) -> Part:
    """Construct part from a config dict.

    :param key: The part's key.
    :param config_dict: The dict with raw part config data.
    :raise ConfigError: If the configuration dict is invalid.
    :raise KeyError: If the config is missing required attributes.
    :return: The version part.
    """
    log.debug(f'Parsing config part <{key}>')
    if config_dict.get('identifier') and config_dict.get('number'):
        raise ConfigError(
            f'Part `{key}` cannot be an identifier and number at the '
            f'same time. Configure either `number` or `identifier` '
            f'attribute')
    elif config_dict.get('identifier'):
        return IdentifierPart(
            key=key,
            value=config_dict['value'],
            strings=config_dict['identifier']['strings'],
            requires=config_dict.get('requires'),
            prefix=config_dict.get('prefix'),
            start=config_dict['identifier'].get('start'))
    elif config_dict.get('number'):
        return NumberPart(
            key=key,
            value=config_dict['value'],
            requires=config_dict.get('requires'),
            prefix=config_dict.get('prefix'),
            label=config_dict['number'].get('label'),
            label_suffix=config_dict['number'].get('label-suffix'),
            start=config_dict['number'].get('start'),
            show_start=config_dict['number'].get('show-start'))
    else:
        # Default if no type configuration is specified.
        return NumberPart(
            key=key,
            value=config_dict['value'],
            requires=config_dict.get('requires'),
            prefix=config_dict.get('prefix'))


def files_from_dict(config_dict: Dict) -> List[FileUpdater]:
    """Get the `files` attribute from a config dict.

    :param config_dict: The dict with raw part config data.
    :raise ConfigError: If the configuration dict is invalid.
    :return: FileUpdater objects based on the `files` attribute.
    """
    try:
        file_updaters: List[FileUpdater] = []
        for file_config in config_dict.get('files', []):
            log.debug(f'Parsing config `files` path <{file_config["path"]}>')
            file_updaters.append(FileUpdater(
                path=file_config['path'],
                patterns=file_config.get('patterns')
            ))
        return file_updaters
    except KeyError as key_error:
        key = key_error.args[0]
        raise ConfigError(
            f'You must have the required attribute `{key}` configured in '
            f'`files`')



class MyverError(Exception):
    """Configuration is invalid."""
    def __init__(self, message: str):
        self.message: str = message
        super().__init__()


class ConfigError(MyverError):
    """Configuration is invalid."""


class BumpError(MyverError):
    """Bumping a version or part failed."""



import re
from dataclasses import dataclass
from glob import glob
from logging import getLogger
from typing import List

from jinja2 import Template

log = getLogger(__name__)


class FileUpdater:
    """Updates files with new versions.

    :param path: The path glob for the files to update.
    :param patterns: The patterns to base off of when updating.
    """

    def __init__(self,
                 path: str,
                 patterns: List[str] = None):
        self.path: str = path
        self.patterns: List[str] = patterns or ['{{ version }}']

    def update(self, old_version: str, new_version: str):
        log.debug(f'Doing update for glob <{self.path}>')
        for path in glob(self.path):
            try:
                log.info(f'Updating <{path}>')
                with open(path, 'r') as file:
                    data = file.read()
                with open(path, 'w') as file:
                    updated = self._update_data(data, old_version, new_version)
                    file.write(updated)
            except FileNotFoundError:
                log.error(f'Path does not exist <{path}>')
            except OSError as e:
                log.error(f'Error {e.errno} updating <{path}>, {e.strerror}')

    def _update_data(self, data: str, old_version: str,
                     new_version: str) -> str:
        rendered_patterns = self._rendered_patterns(old_version)
        update_pairs: List[UpdatePair] = []
        updated_data = data

        for pattern in rendered_patterns:
            log.debug(f'Searching for pattern <{pattern}>')
            for match in re.findall(pattern, data):
                original = match
                updated = match.replace(old_version, new_version)
                log.debug(f'Changing <{original}> to <{updated}>')
                update_pairs.append(UpdatePair(original, updated))

        for pair in update_pairs:
            updated_data = updated_data.replace(pair.original, pair.updated)

        return updated_data

    def _rendered_patterns(self, version: str) -> List[str]:
        rendered_patterns = []

        for pattern in self.patterns:
            log.debug(f'Rendering pattern <{pattern}>')
            template = Template(pattern)
            regex_valid_version = f'{re.escape(version)}'
            rendered = template.render(version=regex_valid_version)
            log.debug(f'Rendered as <{rendered}>')
            rendered_patterns.append(rendered)

        return rendered_patterns

    def __eq__(self, other):
        return (self.path == other.path) and (self.patterns == self.patterns)


@dataclass
class UpdatePair:
    original: str
    updated: str



import setuptools

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='myver',
    version='1.1.0',
    author='Mark Bromell',
    author_email='markbromell.business@gmail.com',
    description='Development tool for configuring and altering software '
                'versions.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/mark-bromell/myver',
    project_urls={
        'Issue Tracker': 'https://github.com/mark-bromell/myver/issues',
        'Source Code': 'https://github.com/mark-bromell/myver',
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    entry_points={
        'console_scripts': [
            'myver = myver.__main__:main',
        ],
    },
    packages=setuptools.find_packages(),
    python_requires='>=3.7',
    install_requires=[
        'ruamel.yaml==0.17.17',
        'jinja2==3.0.3',
    ],
)
